#include "Stereo.h"

void Stereo::LeftEye(void)
{
	float ratio,radians,wd2,ndfl;
	float left,right,top,bottom;
	float m[16];

	if(!glasses)		// ak nepouzivame okuliare
	{
		glViewport(0,0,sirka-(int)posuv,vyska);		// nastavenie obdlznika na obrazovke do ktoreho budeme kreslit
		ratio	= (sirka-(int)posuv) / (float)vyska;	// vypocet pomeru sirky ku vyske
	}
	else	ratio	= screen_x / (float)screen_y;	
		
	radians	= PI180 * fovy / 2;
	wd2		= near_ * (float)tan(radians);
	ndfl	= near_ / focallength;

	// Create the projection
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	left	= - ratio * wd2 + 0.5f * eyesep * ndfl;
	right	=   ratio * wd2 + 0.5f * eyesep * ndfl;
	top		=   wd2;
	bottom	= - wd2;
	glFrustum(left,right,bottom,top,near_,far_);

	glGetFloatv( GL_PROJECTION_MATRIX,m);	// nacitanie matice do pola m
	m[12]= m[0] * 0.5f * eyesep;			// nastavenie posunutia - m[12], vypocet x' = m[0]*x + m[12]
	glLoadMatrixf(m);						// zapisanie matice m do aktualnej projekcnej matice

	glMatrixMode(GL_MODELVIEW);

	if(glasses)
	{
		switch (glassestype)	// Left eye filter
		{
			case REDBLUE:
			case REDGREEN:
			case REDCYAN:	glColor4f(1,0,0,1);break;
			case BLUERED:	glColor4f(0,0,1,1);break;
			case GREENRED:	glColor4f(0,1,0,1);break;
			case CYANRED:	glColor4f(0,1,1,1);break;
		}
	}
}

void Stereo::RightEye(void)
{
	float ratio,radians,wd2,ndfl;
	float left,right,top,bottom;
	float m[16];

	if(!glasses)
	{
		glViewport((int)posuv,0,sirka-(int)posuv,vyska);	// nastavenie obdlznika na obrazovke do ktoreho budeme kreslit
		ratio	= (sirka-(int)posuv) / (float)vyska;	// vypocet pomeru sirky ku vyske
	}
	else	ratio	= screen_x / (float)screen_y;

	radians	= PI180 * fovy / 2;
	wd2		= near_ * (float) tan(radians);
	ndfl	= near_ / focallength;

	// The projection
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	left	= - ratio * wd2 - 0.5f * eyesep * ndfl;
	right	=   ratio * wd2 - 0.5f * eyesep * ndfl;
	top		=   wd2;
	bottom	= - wd2;
	glFrustum(left,right,bottom,top,near_,far_);

	glGetFloatv( GL_PROJECTION_MATRIX,m);	// nacitanie matice do pola m
	m[12]= - m[0] * 0.5f * eyesep;			// nastavenie posunutia - m[12], vypocet x' = m[0]*x + m[12]
	glLoadMatrixf(m);						// zapisanie matice m do aktualnej projekcnej matice

	glMatrixMode(GL_MODELVIEW);

	if(glasses)
	{
		switch (glassestype)	// Right eye filter
		{
			case REDBLUE:	glColor4f(0,0,1,1);break;
			case REDGREEN:	glColor4f(0,1,0,1);break;
			case REDCYAN:	glColor4f(0,1,1,1);break;
			case BLUERED:
			case GREENRED:
			case CYANRED:	glColor4f(1,0,0,1);break;
		}
	}
}

void Stereo::Center(void)
{
	float ratio,radians,wd2;
	float left,right,top,bottom;

	if(!glasses)
	{
		glViewport(0,0,sirka,vyska);	// nastavenie obdlznika na obrazovke do ktoreho budeme kreslit
		ratio	= sirka / (float)vyska;	// vypocet pomeru sirky ku vyske
	}
	else	ratio	= screen_x / (float)screen_y;

	radians	= PI180 * fovy / 2;
	wd2		= near_ * (float) tan(radians);

	// The projection
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	left	= - ratio * wd2;
	right	=   ratio * wd2;
	top		=   wd2;
	bottom	= - wd2;
	glFrustum(left,right,bottom,top,near_,far_);
	glMatrixMode(GL_MODELVIEW);
	glColor4f(1,1,1,1);
}

Stereo::Stereo(bool glasses_)
{
	focallength = 5;				// Focal Length along vd	- zaostrovacia vzdialenost
	sep = 30.0f;
	eyesep  = focallength / sep;	// Eye separation			- separacia oci
	fovy = 45;						// Camera fovy				- uhol medzi dolnou a hornou orezavacou rovinou
	near_ = focallength/5.0f;		// orezanie extremneho stereo
	far_  = 100.0f;					// nastavenie vzdialenej orezavacej roviny
	glassestype = REDCYAN;			// cereveno - zelenomodre okuliare
	posuv = 200;					// posuv laveho a praveho pohladu kvoli videniu bez okuliarov
	glasses = glasses_;
	auto_focus = 1;
}

void Stereo::Set(float vzdialenost)
{
	focallength = vzdialenost;			// Focal Length along vd
	eyesep  = focallength / sep;	// Eye separation
	near_ = focallength/5.0f;			// orezanie extremneho stereo
}

void Stereo::Update(float vzdialenost)
{
	if( (vzdialenost!=0) && auto_focus) Set(vzdialenost);
	if     (keys['W']) { focallength += 5  /fps; eyesep  = focallength / sep; near_ = focallength/5.0f; }
	else if(keys['Q']) { focallength -= 5  /fps; eyesep  = focallength / sep; near_ = focallength/5.0f; }
	if     (keys['X']) { sep   += 10 /fps; eyesep  = focallength / sep; }
	else if(keys['Z']) { sep   -= 10 /fps; eyesep  = focallength / sep; }
	if     (keys['O']) posuv += 100/fps;
	else if(keys['I']) posuv -= 100/fps;
	if     (keys['D']) { glasses = !glasses; keys['D'] = 0; }
	if     (keys['U']) { auto_focus = !auto_focus; keys['U'] = 0; }
}
